<?php

namespace Drupal\challenge_mitigation;

use Drupal\challenge_mitigation\Form\ChallengeMitigationIpFilterForm;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityListBuilder;
use Drupal\Core\Form\FormBuilder;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * List builder for CM whitelist IP entities.
 *
 * Displays IP addresses that have been temporarily whitelisted
 * after passing the challenge. Includes metadata such as origin,
 * creation date, expiration date, and basic operations.
 */
class ChallengeMitigationWhitelistIpListBuilder extends EntityListBuilder {

  /**
   * The form builder service.
   *
   * @var \Drupal\Core\Form\FormBuilder
   */
  private FormBuilder $formBuilder;

  /**
   * Constructs a new list builder.
   */
  public function __construct(
    $entityType,
    $storage,
    protected DateFormatterInterface $dateFormatter,
    private readonly RequestStack $requestStack,
    FormBuilder $formBuilder,
    protected int $whitelistDurationMinutes,
  ) {
    parent::__construct($entityType, $storage);
    $this->formBuilder = $formBuilder;
  }

  /**
   * {@inheritdoc}
   *
   * Injects services from the container.
   */
  public static function createInstance(ContainerInterface $container, $entityType): static {
    $config = $container->get('config.factory')->get('challenge_mitigation.settings');
    return new static(
      $entityType,
      $container->get('entity_type.manager')->getStorage($entityType->id()),
      $container->get('date.formatter'),
      $container->get('request_stack'),
      $container->get('form_builder'),
      (int) ($config->get('whitelist_duration') ?? 1440),
    );
  }

  /**
   * {@inheritdoc}
   *
   * Defines the table headers used in the admin list.
   */
  public function buildHeader(): array {
    $header['ip'] = $this->t('IP address');
    $header['origin'] = $this->t('Origin');
    $header['created'] = $this->t('Created');
    $header['expiration'] = $this->t('Expiration');
    $header['operations'] = $this->t('Operations');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   *
   * Defines how each row of the list is rendered.
   */
  public function buildRow(EntityInterface $entity): array {
    $created = (int) $entity->get('created')->value;
    $expirationTimestamp = $created + ($this->whitelistDurationMinutes * 60);
    $row['ip'] = $entity->get('ip')->value;
    $row['origin'] = $entity->get('origin')->value;
    $row['created'] = $this->dateFormatter->format($created, 'short');
    $row['expiration'] = $this->dateFormatter->format($expirationTimestamp, 'short');
    return $row + parent::buildRow($entity);
  }

  /**
   * {@inheritdoc}
   */
  public function render() {
    $build['form'] = $this->formBuilder->getForm(ChallengeMitigationIpFilterForm::class);
    $build += parent::render();
    return $build;
  }

  /**
   * {@inheritdoc}
   *
   * Enables pagination by limiting the number of items loaded.
   */
  public function load(): array {
    $request = $this->requestStack->getCurrentRequest();
    $ip_filter = $request->query->get('ip_filter') ?? '';
    $query = $this->getStorage()->getQuery()
      ->accessCheck(TRUE)
      ->pager(50);

    if (!empty($ip_filter)) {
      $query->condition('ip', '%' . $ip_filter . '%', 'LIKE');
    }

    $ids = $query->execute();
    return $this->storage->loadMultiple($ids);
  }

}
