<?php

namespace Drupal\challenge_mitigation\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the Challenge Mitigation Whitelist IP entity.
 *
 * This entity stores IP addresses that have passed the challenge
 * and are temporarily whitelisted. Each record includes:
 * - The IP address (string)
 * - The origin or method used to whitelist the IP (optional)
 * - A timestamp of creation.
 *
 * @ContentEntityType(
 *   id = "cm_whitelist_ip",
 *   label = @Translation("Challenge Mitigation Whitelist IP"),
 *   base_table = "cm_whitelist_ip",
 *   entity_keys = {
 *     "id" = "id",
 *     "uuid" = "uuid",
 *     "label" = "ip"
 *   },
 *   handlers = {
 *     "list_builder" = "Drupal\challenge_mitigation\ChallengeMitigationWhitelistIpListBuilder",
 *     "form" = {
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm"
 *     },
 *     "access" = "Drupal\Core\Entity\EntityAccessControlHandler"
 *   },
 *   admin_permission = "administer challenge mitigation whitelist",
 *   links = {
 *     "collection" = "/admin/config/challenge-mitigation/whitelist_ips",
 *     "delete-form" = "/admin/config/challenge-mitigation/whitelist_ips/{cm_whitelist_ip}/delete"
 *   }
 * )
 */
class ChallengeMitigationWhitelistIp extends ContentEntityBase {

  /**
   * Defines the base fields used in the whitelist IP entity.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entityType
   *   The entity type definition.
   *
   * @return \Drupal\Core\Field\BaseFieldDefinition[]
   *   An array of field definitions.
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entityType): array {
    $fields = [];

    // Primary internal ID field.
    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('ID'))
      ->setReadOnly(TRUE);

    // UUID field for entity identification.
    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('UUID'))
      ->setReadOnly(TRUE);

    // IP address field (IPv4 or IPv6).
    $fields['ip'] = BaseFieldDefinition::create('string')
      ->setLabel(t('IP address'))
      ->setRequired(TRUE)
      ->setSettings([
    // Supports full IPv6.
        'max_length' => 45,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    // Optional origin field (e.g. "Challenge form", "Drush import"...).
    $fields['origin'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Origin'))
      ->setRequired(FALSE)
      ->setSettings([
        'max_length' => 255,
        'text_processing' => 0,
      ])
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 1,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 1,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    // Created timestamp to determine expiry.
    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the whitelist entry was created.'))
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'timestamp',
        'weight' => 2,
      ])
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

}
