<?php

namespace Drupal\challenge_mitigation\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\PrivateKey;
use Drupal\Core\Render\BareHtmlPageRendererInterface;
use Drupal\challenge_mitigation\Form\ChallengeMitigationAccessForm;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\path_alias\AliasManager;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\IpUtils;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Enforces CAPTCHA or JS challenges for anonymous visitors.
 *
 * This subscriber checks incoming requests against configured URL patterns.
 * If the current IP or User-Agent is not whitelisted, the user is served
 * a lightweight challenge form.
 */
class ChallengeMitigationSubscriber implements EventSubscriberInterface {

  /**
   * The immutable configuration for this form.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private readonly ImmutableConfig $config;

  public function __construct(
    protected ConfigFactoryInterface $configFactory,
    protected CurrentPathStack $currentPath,
    protected EntityTypeManagerInterface $entityTypeManager,
    protected PathMatcherInterface $pathMatcher,
    protected AliasManager $pathAliasManager,
    protected FormBuilderInterface $formBuilder,
    protected BareHtmlPageRendererInterface $bareRenderer,
    protected KillSwitch $killSwitch,
    protected AccountProxyInterface $currentUser,
    protected PrivateKey $private_key,
  ) {
    // Load config once for performance.
    $this->config = $this->configFactory->get('challenge_mitigation.settings');
  }

  /**
   * Kernel request entrypoint.
   *
   * Renders the challenge form instead of continuing the page load.
   */
  public function onKernelRequest(RequestEvent $event): void {
    $request = $event->getRequest();

    if (!$this->shouldProcess($event)) {
      return;
    }

    $ip = $request->getClientIp();
    $uri = $request->getRequestUri();
    $userAgent = $request->headers->get('User-Agent') ?? '';

    if (!$this->shouldChallengeUri($uri)) {
      return;
    }

    if ($this->shouldSkipAuthenticatedUsers()) {
      return;
    }

    if ($this->isCookieBypassValid($request)) {
      return;
    }

    if ($this->isWhitelistedIp($ip)) {
      return;
    }

    if ($this->isWhitelistedUserAgent($userAgent)) {
      return;
    }

    $request->getSession()->set('cm_return_url', $uri);

    $form = $this->formBuilder->getForm(ChallengeMitigationAccessForm::class);

    $build = [
      'form' => $form,
    ];

    $response = $this->bareRenderer->renderBarePage(
      $build,
      'Challenge Access Verification',
      'challenge_mitigation_page'
    );

    $this->killSwitch->trigger();
    $event->setResponse($response);
  }

  /**
   * Checks if the user has a valid bypass cookie.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current HTTP request.
   *
   * @return bool
   *   TRUE if bypass cookie is valid, FALSE otherwise.
   */
  private function isCookieBypassValid(Request $request): bool {
    if (!$this->config->get('use_cookie_bypass')) {
      return FALSE;
    }

    $cookie_value = $request->cookies->get('ChallengeMitigationBypass');
    if (!$cookie_value) {
      return FALSE;
    }

    [$timestamp, $hmac] = explode('.', $cookie_value, 2) + [NULL, NULL];
    if (!$timestamp || !$hmac) {
      return FALSE;
    }

    $expected_hmac = hash_hmac('sha256', $timestamp, $this->private_key->get());
    if (!hash_equals($expected_hmac, $hmac)) {
      return FALSE;
    }

    $lifetime = (int) ($this->config->get('cookie_bypass_lifetime') ?? 2592000);
    if ((time() - (int) $timestamp) > $lifetime) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Determines if authenticated users should skip challenge.
   */
  private function shouldSkipAuthenticatedUsers(): bool {
    if ($this->config->get('skip_authenticated_users') && $this->currentUser->isAuthenticated()) {
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Determines whether this subscriber should handle the request.
   */
  private function shouldProcess(RequestEvent $event): bool {
    $request = $event->getRequest();

    if (!$event->isMainRequest() || $request->isXmlHttpRequest()) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Checks if the current request URI matches challenge targets.
   */
  private function shouldChallengeUri(string $uri): bool {
    if (!$this->config->get('enabled')) {
      return FALSE;
    }

    if ($this->config->get('challenge_everywhere')) {
      return TRUE;
    }

    $patterns = array_filter(array_map('trim', explode("\n", $this->config->get('challenge_urls') ?? '')));
    return $this->matchUriAgainstPatterns($uri, $patterns);
  }

  /**
   * Checks if the IP is manually or dynamically whitelisted.
   */
  private function isWhitelistedIp(string $ip): bool {
    $manualList = array_filter(array_map('trim', explode("\n", $this->config->get('manual_ip_whitelist') ?? '')));
    if (!empty($manualList) && IpUtils::checkIp($ip, $manualList)) {
      return TRUE;
    }

    $storage = $this->entityTypeManager->getStorage('cm_whitelist_ip');
    $results = $storage->getQuery()
      ->accessCheck(FALSE)
      ->condition('ip', $ip)
      ->range(0, 1)
      ->execute();

    return !empty($results);
  }

  /**
   * Checks if the current User-Agent matches a whitelisted pattern.
   */
  private function isWhitelistedUserAgent(string $userAgent): bool {
    $agents = array_filter(array_map('trim', explode("\n", $this->config->get('manual_user_agent_whitelist') ?? '')));
    foreach ($agents as $pattern) {
      if (@preg_match($pattern, '') === FALSE) {
        // Skip invalid regex.
        continue;
      }
      if (preg_match($pattern, $userAgent)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * Matches a URI against wildcard path patterns.
   */
  private function matchUriAgainstPatterns(string $uri, array $patterns): bool {
    foreach ($patterns as $pattern) {
      $pattern = trim((string) $pattern);
      if ($pattern === '') {
        continue;
      }

      $regex = '#^' . str_replace('\*', '.*', preg_quote($pattern, '#')) . '$#u';
      if (preg_match($regex, $uri)) {
        return TRUE;
      }
    }

    return FALSE;
  }

  /**
   * Registers event handler.
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::REQUEST => ['onKernelRequest', 100],
    ];
  }

}
