<?php

namespace Drupal\challenge_mitigation\Form;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\Messenger;
use Drupal\Core\PrivateKey;
use Drupal\Core\Url;
use Drupal\challenge_mitigation\Entity\ChallengeMitigationWhitelistIp;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

/**
 * Public-facing challenge form to whitelist user IPs.
 */
class ChallengeMitigationAccessForm extends FormBase {

  /**
   * The immutable config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private readonly ImmutableConfig $config;

  /**
   * Constructs a new ChallengeMitigationAccessForm.
   */
  public function __construct(
    RequestStack $requestStack,
    ConfigFactoryInterface $configFactory,
    Messenger $messenger,
    protected DateFormatterInterface $dateFormatter,
    protected ModuleHandlerInterface $moduleHandler,
    protected TimeInterface $time,
    protected PrivateKey $privateKey,
  ) {
    $this->requestStack = $requestStack;
    $this->configFactory = $configFactory;
    $this->messenger = $messenger;
    $this->config = $this->configFactory->get('challenge_mitigation.settings');
  }

  /**
   * Dependency injection factory.
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('request_stack'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('date.formatter'),
      $container->get('module_handler'),
      $container->get('datetime.time'),
      $container->get('private_key')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'challenge_mitigation_access_form';
  }

  /**
   * Builds the challenge form.
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $session = $this->requestStack->getCurrentRequest()->getSession();
    $form['#theme'] = 'challenge_mitigation_form';

    if (!$session->get('cm_return_url')) {
      $form['#markup'] = $this->t('Invalid challenge mitigation code.');
      return $form;
    }

    $request = $this->requestStack->getCurrentRequest();
    $challengeMode = $this->determineChallengeMode($request);
    $useCaptcha = $this->config->get('use_captcha_module') ?? FALSE;
    $captchaEnabled = $this->moduleHandler->moduleExists('captcha');

    if ($challengeMode === 'automatic_js') {
      $form['#attached']['library'][] = 'challenge_mitigation/challenge_js';
    }

    if ($challengeMode === 'hard' && $useCaptcha && $captchaEnabled) {
      $form['captcha'] = [
        '#type' => 'captcha',
        '#captcha_type' => 'default',
      ];
    }

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Validate access'),
      '#attributes' => $challengeMode === 'automatic_js'
        ? ['style' => 'display:none;']
        : [],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * On submit, adds the visitor's IP to the whitelist.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $ip = $this->requestStack->getCurrentRequest()->getClientIp();

    $entity = ChallengeMitigationWhitelistIp::create([
      'ip' => $ip,
      'origin' => 'Default challenge mitigation',
      'created' => $this->time->getCurrentTime(),
    ]);
    $entity->save();

    if ($this->config->get('use_cookie_bypass')) {
      $response = new Response();
      $this->setBypassCookie($response);
      $response->sendHeaders();
    }

    $session = $this->requestStack->getCurrentRequest()->getSession();
    $origin = $session->get('cm_return_url') ?? '/';
    $session->remove('cm_return_url');

    $this->messenger->deleteAll();
    $form_state->setRedirectUrl(Url::fromUserInput($origin));
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state): void {
    parent::validateForm($form, $form_state);

    // Optional: display messages if needed in the form output.
    $form['messages'] = [
      '#type' => 'container',
      'status_messages' => [
        '#theme' => 'status_messages',
        '#message_list' => $this->messenger->all(),
        '#status_headings' => [
          'status' => $this->t('Status message'),
          'error' => $this->t('Error message'),
          'warning' => $this->t('Warning message'),
        ],
      ],
    ];
  }

  /**
   * Determines the challenge mode to use for the current visitor.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current HTTP request.
   *
   * @return string
   *   The challenge mode: 'automatic_js' or 'hard'.
   */
  private function determineChallengeMode(Request $request): string {
    $challenge_mode = $this->config->get('challenge_mode') ?? 'automatic_js';
    if ($challenge_mode === 'adaptive_hard') {
      $force_hard_challenge = FALSE;
      $user_agent = $request->headers->get('User-Agent') ?? '';
      $patterns = preg_split('/\r\n|\r|\n/', $this->config->get('adaptive_suspect_user_agents') ?? '');
      foreach ($patterns as $pattern) {
        if (trim($pattern) === '') {
          continue;
        }
        if (@preg_match($pattern, $user_agent)) {
          $force_hard_challenge = TRUE;
          break;
        }
      }
      return $force_hard_challenge ? 'hard' : 'automatic_js';
    }
    return $challenge_mode;
  }

  /**
   * Sets the signed bypass cookie after successful challenge.
   *
   * @param \Symfony\Component\HttpFoundation\Response $response
   *   The response object where the cookie should be attached.
   */
  private function setBypassCookie(Response $response): void {
    if (!$this->config->get('use_cookie_bypass')) {
      return;
    }

    $timestamp = time();
    $signature = hash_hmac('sha256', (string) $timestamp, $this->privateKey->get());
    $cookie_value = $timestamp . '.' . $signature;

    $lifetime = (int) ($this->config->get('cookie_bypass_lifetime') ?? 2592000);
    $expire = $timestamp + $lifetime;

    $cookie = Cookie::create(
      'ChallengeMitigationBypass',
      $cookie_value,
      $expire,
      '/',
      '',
      TRUE,
      TRUE,
      FALSE,
      Cookie::SAMESITE_LAX
    );

    $response->headers->setCookie($cookie);
  }

}
