<?php

namespace Drupal\challenge_mitigation\Form;

use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Settings form for Challenge Mitigation behavior.
 *
 * Stores its data in configuration: challenge_mitigation.settings.
 */
class ChallengeMitigationSettingsForm extends FormBase {

  public function __construct(
    protected ModuleHandlerInterface $moduleHandler,
  ) {
  }

  /**
   * Dependency injection factory.
   */
  public static function create(ContainerInterface $container): static {
    return new static(
      $container->get('module_handler'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'cm_settings_form';
  }

  /**
   * Builds the Challenge Mitigation settings form.
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('challenge_mitigation.settings');
    $captchaAvailable = $this->moduleHandler->moduleExists('captcha');

    $form['intro'] = [
      '#type' => 'markup',
      '#markup' => $this->t('Based on these settings, users visiting the site or accessing one of the listed URLs will need to complete a captcha challenge once. After passing, their IP will be temporarily whitelisted to bypass further challenges.'),
      '#prefix' => '<div class="description form-item">',
      '#suffix' => '</div>',
    ];

    $form['enabled'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable challenge mitigation'),
      '#description' => $this->t('Activates the challenge mitigation system.'),
      '#default_value' => $config->get('enabled'),
    ];

    $form['challenge_everywhere'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable challenge for the entire site'),
      '#description' => $this->t('If enabled, captcha is required site-wide, ignoring specific URLs.'),
      '#default_value' => $config->get('challenge_everywhere'),
    ];

    $form['challenge_urls'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Challenge URLs'),
      '#description' => $this->t('List one internal path per line (e.g. /user, /checkout). Use &lt;front&gt; for the homepage. These are the URLs where captcha will be enforced unless global challenge is enabled.'),
      '#default_value' => $config->get('challenge_urls') ?? "",
    ];

    $form['use_captcha_module'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Use CAPTCHA module integration'),
      '#description' => $captchaAvailable
        ? $this->t('Enable integration with the CAPTCHA module for challenge display. (Only in hard mode challenge)')
        : $this->t('The CAPTCHA module is not enabled. Install and enable it to use this feature.'),
      '#default_value' => $captchaAvailable ? $config->get('use_captcha_module') : FALSE,
      '#disabled' => !$captchaAvailable,
    ];

    $form['challenge_mode'] = [
      '#type' => 'select',
      '#title' => $this->t('Challenge mode'),
      '#description' => $this->t('Choose how the challenge should be enforced for visitors.'),
      '#options' => [
        'automatic_js' => $this->t('Automatic JS - Transparent challenge using JavaScript.'),
        'hard' => $this->t('Hard - User must manually submit the challenge form.'),
        'adaptive_hard' => $this->t('Adaptive Hard - Force hard challenge for suspicious User-Agents, automatic JS otherwise.'),
      ],
      '#default_value' => $config->get('challenge_mode') ?? 'automatic_js',
    ];

    $form['adaptive_suspect_user_agents'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Suspicious User-Agent patterns'),
      '#description' => $this->t('Enter one regular expression per line. If a visitor User-Agent matches any of these patterns, a Hard Challenge will be forced if Adaptive Hard mode is enabled.'),
      '#default_value' => $config->get('adaptive_suspect_user_agents') ?? "/curl/\n/wget/\n/bot/\n/crawler/\n/python-requests/",
    ];

    $form['whitelist_duration'] = [
      '#type' => 'number',
      '#title' => $this->t('Whitelist duration (in minutes)'),
      '#description' => $this->t('Defines how long (in minutes) an IP stays whitelisted after passing the captcha.'),
      '#min' => 1,
      '#default_value' => $config->get('whitelist_duration') ?? 1440,
    ];

    $form['skip_authenticated_users'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Skip authenticated users'),
      '#description' => $this->t('If enabled, authenticated users will not be challenged. Only anonymous visitors will have to pass the challenge.'),
      '#default_value' => $config->get('skip_authenticated_users') ?? FALSE,
    ];

    $form['use_cookie_bypass'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable cookie-based challenge bypass'),
      '#description' => $this->t('If enabled, a signed cookie will be set after a successful challenge, allowing users to bypass future challenges from the same browser.'),
      '#default_value' => $config->get('use_cookie_bypass') ?? FALSE,
    ];

    $form['cookie_bypass_lifetime'] = [
      '#type' => 'number',
      '#title' => $this->t('Cookie bypass lifetime (in seconds)'),
      '#description' => $this->t('Defines how long (in seconds) the bypass cookie remains valid after being issued.'),
      '#min' => 60,
      '#default_value' => $config->get('cookie_bypass_lifetime') ?? 2592000,
    ];

    $form['manual_ip_whitelist'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Manually whitelisted IPs'),
      '#description' => $this->t('Enter one IP address per line that should always bypass the challenge. IPv4, IPv6 and CIDR notation supported.'),
      '#default_value' => $config->get('manual_ip_whitelist') ?? '',
    ];

    $form['manual_user_agent_whitelist'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Manually whitelisted User-Agents'),
      '#description' => $this->t('Enter one regular expression per line to whitelist User-Agent strings. These are PHP-compatible regex patterns (e.g. /Safari/, /^Mozilla.*Chrome/). Useful for trusted crawlers or internal services.'),
      '#default_value' => $config->get('manual_user_agent_whitelist') ?? '',
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Save settings'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   *
   * Saves form values to configuration.
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {
    $this->configFactory()->getEditable('challenge_mitigation.settings')
      ->set('enabled', $form_state->getValue('enabled'))
      ->set('challenge_everywhere', $form_state->getValue('challenge_everywhere'))
      ->set('challenge_urls', $form_state->getValue('challenge_urls'))
      ->set('use_captcha_module', $form_state->getValue('use_captcha_module'))
      ->set('challenge_mode', $form_state->getValue('challenge_mode'))
      ->set('whitelist_duration', $form_state->getValue('whitelist_duration'))
      ->set('manual_ip_whitelist', $form_state->getValue('manual_ip_whitelist'))
      ->set('manual_user_agent_whitelist', $form_state->getValue('manual_user_agent_whitelist'))
      ->set('skip_authenticated_users', $form_state->getValue('skip_authenticated_users'))
      ->set('use_cookie_bypass', $form_state->getValue('use_cookie_bypass'))
      ->set('cookie_bypass_lifetime', $form_state->getValue('cookie_bypass_lifetime'))
      ->set('adaptive_suspect_user_agents', $form_state->getValue('adaptive_suspect_user_agents'))
      ->save();

    $this->messenger()->addStatus($this->t('Settings saved.'));
  }

}
